/*
 * Copyright (c) 2002-2016 Gargoyle Software Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.gargoylesoftware.htmlunit.html;

import static com.gargoylesoftware.htmlunit.BrowserVersionFeatures.CSS_KEYGEN_DISPLAY_INLINE_ALWAYS;
import static com.gargoylesoftware.htmlunit.BrowserVersionFeatures.CSS_KEYGEN_DISPLAY_INLINE_JS;

import java.util.Map;

import com.gargoylesoftware.htmlunit.SgmlPage;

/**
 * Wrapper for the HTML element "keygen".
 *
 * @author Ronald Brill
 * @author Ahmed Ashour
 */
public class HtmlKeygen extends HtmlElement {

    /** The HTML tag represented by this element. */
    public static final String TAG_NAME = "keygen";

    private boolean createdByJavascript_;

    /**
     * Creates a new instance.
     *
     * @param qualifiedName the qualified name of the element type to instantiate
     * @param page the page that contains this element
     * @param attributes the initial attributes
     */
    HtmlKeygen(final String qualifiedName, final SgmlPage page,
            final Map<String, DomAttr> attributes) {
        super(qualifiedName, page, attributes);
    }

    /**
     * <span style="color:red">INTERNAL API - SUBJECT TO CHANGE AT ANY TIME - USE AT YOUR OWN RISK.</span><br>
     *
     * Marks this frame as created by javascript. This is needed to handle
     * some special IE behavior.
     */
    public void markAsCreatedByJavascript() {
        createdByJavascript_ = true;
    }

    /**
     * <span style="color:red">INTERNAL API - SUBJECT TO CHANGE AT ANY TIME - USE AT YOUR OWN RISK.</span><br>
     *
     * Returns true if this frame was created by javascript. This is needed to handle
     * some special IE behavior.
     * @return true or false
     */
    public boolean wasCreatedByJavascript() {
        return createdByJavascript_;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public DisplayStyle getDefaultStyleDisplay() {
        if (hasFeature(CSS_KEYGEN_DISPLAY_INLINE_ALWAYS)) {
            return DisplayStyle.INLINE;
        }
        if (wasCreatedByJavascript()) {
            if (getParentNode() == null) {
                if (hasFeature(CSS_KEYGEN_DISPLAY_INLINE_JS)) {
                    return DisplayStyle.BLOCK;
                }
                return DisplayStyle.EMPTY;
            }
            if (hasFeature(CSS_KEYGEN_DISPLAY_INLINE_JS)) {
                return DisplayStyle.INLINE;
            }
        }
        return DisplayStyle.INLINE_BLOCK;
    }
}
